//-----------------------------------------------------------------------------
// Torque Game Builder
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

if (!isObject(MouseOverGrowBehavior))
{
   %template = new BehaviorTemplate(MouseOverGrowBehavior);
   
   %template.friendlyName = "Mouse Over Grow";
   %template.behaviorType = "GUI";
   %template.description  = "Grow the object when the mouse hovers over it";

   %template.addBehaviorField(scaleTime, "The amount of time to animate the scale (seconds)", float, 0.25);
   %template.addBehaviorField(scaleFactor, "Over-ridden (The scale factor of the object)", float, 1.2);
}

function MouseOverGrowBehavior::onBehaviorAdd(%this)
{
   %this.owner.setUseMouseEvents(true);
   %this.owner.enableUpdateCallback();
   
   %this.startSize = %this.owner.size;
   %this.targetSize = %this.startSize;
   %this.scaleFactor = %this.owner.scaleFactor;//added to change scale factor per owner
   %this.scaleAmount = t2dVectorSub(t2dVectorScale(%this.startSize, %this.scaleFactor), %this.startSize);

//   echo(%this.owner.scaleFactor);
   
}

function MouseOverGrowBehavior::onMouseEnter(%this, %modifier, %worldPos)
{
//   $saveframe=%this.owner.frame;
   %this.targetSize = t2dVectorScale(%this.startSize, %this.scaleFactor);
   %this.owner.grow = true;//used in teleport
}

function MouseOverGrowBehavior::onMouseLeave(%this, %modifier, %worldPos)
{
//   %this.owner.frame=$saveframe;
   %this.targetSize = %this.startSize;
   %this.owner.grow = false;
}

function MouseOverGrowBehavior::onUpdate(%this)
{
   %linked = isObject(%this.owner.linked);
   if (%linked){
      %linkedX = %this.owner.linked.size.x;
      %linkedY = %this.owner.linked.size.y;
//      echo("MouseOverGrowBehavior::onUpdate:"@%this.owner.linked.text);
   }
        
   %currentX = %this.owner.size.x;
   %targetX = %this.targetSize.x;
   %scaleX = %this.scaleAmount.x;
   
   %currentY = %this.owner.size.y;
   %targetY = %this.targetSize.y;
   %scaleY = %this.scaleAmount.y;
   
   if ((%currentX == %targetX) && (%currentY == %targetY))
      return;
   
   if (%currentX < %targetX)
   {
      %this.owner.setSizeX(%currentX + ((0.032 / %this.scaleTime) * %scaleX));
      if (%linked) %this.owner.linked.setSizeX(%linkedX + ((0.032 / %this.scaleTime) * %scaleX)); 
      if (%this.owner.getSizeX() > %targetX)
         %this.owner.setSizeX(%targetX);
   }
   else if (%currentX > %targetX)
   {
      %this.owner.setSizeX(%currentX - ((0.032 / %this.scaleTime) * %scaleX));
      if (%this.owner.getSizeX() < %targetX)
         %this.owner.setSizeX(%targetX);

      if (%linked) {
         %this.owner.linked.setSizeX(%linkedX - ((0.032 / %this.scaleTime) * %scaleX));
         if (%this.owner.linked.getSizeX() < %targetX)
            %this.owner.linked.setSizeX(%targetX);
      }
   }
   
   if (%currentY < %targetY)
   {
      %this.owner.setSizeY(%currentY + ((0.032 / %this.scaleTime) * %scaleY));
      if (%linked) %this.owner.linked.setSizeY(%linkedY + ((0.032 / %this.scaleTime) * %scaleY));
      if (%this.owner.getSizeY() > %targetY)
         %this.owner.setSizeY(%targetY);
   }
   else if (%currentY > %targetY)
   {
      %this.owner.setSizeY(%currentY - ((0.032 / %this.scaleTime) * %scaleY));
      if (%this.owner.getSizeY() < %targetY)
         %this.owner.setSizeY(%targetY);

      if (%linked){
         %this.owner.linked.setSizeY(%linkedY - ((0.032 / %this.scaleTime) * %scaleY));
         if (%this.owner.linked.getSizeY() < %targetY)
            %this.owner.linked.setSizeY(%targetY);
      }
   }
}
